/*
 * bltMacOSXDraw.c --
 */

#include <bltInt.h>

extern void macosxDrawRotatedText(Display*, Drawable, GC, Tk_Font,
				  double, double, double, char*);

int Blt_DrawRotatedText(Display *display, Drawable drawable, int x, int y,
			double angle, TextStyle *tsPtr, TextLayout *textPtr)
{
  double theta, sinTheta, cosTheta;
  Point2D center;
  int bbWidth, bbHeight;
  double rotWidth, rotHeight;
  int xx,yy;
  TextFragment *fragPtr, *endPtr;

  Blt_GetBoundingBox(textPtr->width, textPtr->height, angle, &rotWidth, 
		     &rotHeight, (Point2D *)NULL);
  bbWidth = ROUND(rotWidth);
  bbHeight = ROUND(rotHeight);
  Blt_TranslateAnchor(x, y, bbWidth, bbHeight, tsPtr->anchor, &xx, &yy);

  theta = angle/180*M_PI;
  center.x = textPtr->width/2.;
  center.y = textPtr->height/2.;

  sinTheta = sin(theta);
  cosTheta = cos(theta);

  endPtr = textPtr->fragArr + textPtr->nFrags;

  for (fragPtr = textPtr->fragArr; fragPtr < endPtr; fragPtr++) {
    char* text = textPtr->fragArr->text;
    Point2D p,q,r,v;

    /*
      Vector q = Vector(fragPtr) * 
      Translate(-center) * 
      Rotate(theta) * 
      Translate(bb) * 
      Translate(xx,yy);
    */

    p.x = -center.x + (double)fragPtr->x;
    p.y = -center.y + (double)fragPtr->y;

    q.x =  (p.x * cosTheta) + (p.y * sinTheta);
    q.y = -(p.x * sinTheta) + (p.y * cosTheta);

    r.x = q.x + bbWidth/2.;
    r.y = q.y + bbHeight/2.;

    v.x =  r.x + xx;
    v.y =  r.y + yy;

    macosxDrawRotatedText(display, drawable, tsPtr->gc, tsPtr->font, 
			  v.x, v.y, theta, text);
  }

  return 1;
}

void XPutImage(Display* display, Drawable d, GC gc, XImage* image,
	       int src_x, int src_y, int dest_x, int dest_y,
	       unsigned int width, unsigned int height)
{
  TkPutImage(NULL, 0, display, d, gc, image, src_x, src_y,
	     dest_x, dest_y, width, height);
}

void XFreePixmap(Display* display, Pixmap pixmap)
{
   Tk_FreePixmap(display, pixmap);
}

int XFree(void* data) 
{
  if ((data) != NULL) 
    ckfree((char *) (data));
  return 1;
}

VisualID XVisualIDFromVisual(Visual* visual)
{
  return visual->visualid;
}

/* no ops */
void XFlush(Display* display) {}
void XReparentWindow(Display* display, Window w, Window parent, int x, int y) {}
void XLowerWindow(Display* display, Window w) {}
void XFreeFont(Display* display,  XFontStruct* font_struct) {}

XFontStruct* XLoadQueryFont(Display* display, char* name) 
{return NULL;}

Bool XGetFontProperty(XFontStruct *font_struct, Atom atom,
		      unsigned long *value_return)
{return 0;}

extern Window XCreateWindow(Display *display, Window parent, 
				int x, int y,
				unsigned int width, unsigned int height,
				unsigned int border_width, int depth,
				unsigned int class, Visual *visual,
				unsigned long valuemask,
				XSetWindowAttributes *attributes)
{return 0;}

Status XAllocColorCells(Display* display, Colormap colormap, 
			Bool contig, 
			unsigned long plane_masks_return, 
			unsigned int nplanes, 
			unsigned long pixels_return, 
			unsigned int npixels)
{return 0;}



