#define Cursor XCursor
#define Region XRegion

#include <tcl.h>
#include <tk.h>

#undef Cursor
#undef Region

#include "../../include/tkMacOSX.h"
#include "../../include/tkMacOSXInt.h"
#include "../../include/tkMacOSXFont.h"
#include <Carbon/Carbon.h>

#define RGBFLOATRED(c)   ((c).red   / 65535.0)
#define RGBFLOATGREEN(c) ((c).green / 65535.0)
#define RGBFLOATBLUE(c)  ((c).blue  / 65535.0)

void macosxDrawRotatedText(Display *display, Drawable drawable, 
			   GC gc, Tk_Font font, 
			   double x, double y, double theta, char* text)
{
  unsigned long color = gc->foreground;
  MacDrawable* macWin = (MacDrawable*)drawable;

  GWorldPtr destPort = TkMacOSXGetDrawablePort(drawable);
  CGContextRef context;
  OSStatus err = QDBeginCGContext(destPort, &context);

  RgnHandle clipRgn = NewRgn();
  Rect bounds;
  GetPortBounds(destPort, &bounds);

  RectRgn(clipRgn, &bounds);
  SectRegionWithPortClipRegion(destPort, clipRgn);
  SectRegionWithPortVisibleRegion(destPort, clipRgn);

  if (gc->clip_mask) {
    TkpClipMask* msk = (TkpClipMask*)gc->clip_mask;
    if (msk->type == TKP_CLIP_REGION) {
      int xOffset = macWin->xOff + gc->clip_x_origin;
      int yOffset = macWin->yOff + gc->clip_y_origin;
      RgnHandle rgn = NewRgn();
      CopyRgn((RgnHandle)msk->value.region, rgn);
      OffsetRgn(rgn, xOffset, yOffset);
      SectRgn(rgn, clipRgn, clipRgn);
      DisposeRgn(rgn);
    }
  }

  ClipCGContextToRegion(context, &bounds, clipRgn);
  DisposeRgn(clipRgn);
  SyncCGContextOriginWithPort(context, destPort);

  CGAffineTransform mm = CGAffineTransformMakeRotation(theta);
  CGContextSetTextMatrix(context, mm);

  RGBColor macColor;
  if (TkSetMacColor(gc->foreground, &macColor) == true) {
    CGContextSetRGBFillColor(context,
			     RGBFLOATRED(macColor), 
			     RGBFLOATGREEN(macColor), 
			     RGBFLOATBLUE(macColor),
			     1);
    CGContextSetRGBStrokeColor(context,
			       RGBFLOATRED(macColor),
			       RGBFLOATGREEN(macColor),
			       RGBFLOATBLUE(macColor),
			       1);
  }
  CGContextSetTextDrawingMode (context, kCGTextFill);

  Tcl_DString ds;
  Tcl_DStringInit(&ds);
  int size = Tk_PostscriptFontName(gc->font, &ds);
  CGContextSelectFont(context, Tcl_DStringValue(&ds), 
		      size, kCGEncodingMacRoman);
  Tcl_DStringFree(&ds);

  CGContextShowTextAtPoint (context, x, y, text, strlen(text));

  CGContextSynchronize(context);
  QDEndCGContext(destPort, &context);
}



