#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CATVOTRead {t fn} {
    upvar #0 $t T
    global $t

    global debug
    if {$debug(tcl,cat)} {
	puts "CATVOTRead"
    }

    if {$fn != {}} {
	catch {
	    set fp [open $fn r]
	    CATVOTReader $t $fp
	    close $fp
	}
    }
}

proc CATVOTReader {t fp} {
    upvar #0 $t T
    global $t

    # init db
    set T(Nrows) 0
    set T(Ncols) 0
    set T(Header) {}
    set T(HLines) 0
}
    
proc CATVOTFile {fn} {
    CATDialog catoth {} {} {} none
    if {$fn != {}} {
	CATLoadFn $varname $fn CATVOTRead
    }
}

proc CATVOTURL {url} {
    if {[string length $url] == 0} {
	return
    }

    ParseURL $url r
    switch -- $r(scheme) {
	ftp {CATVOTFTP $r(authority) $r(path)}
	file {CATVOTFile $r(path) {}}
	http -
	default {CATVOTHTTP $url}
    }
}

proc CATVOTFTP {host path} {
    global loadParam
    global ds9
    global debug

    set ftp [ftp::Open $host "ftp" "-ds9@" -mode passive]
    if {$ftp > -1} {
	set fn "$ds9(tmpdir)/[file tail $path]"
	set ftp::VERBOSE $debug(tcl,ftp)
	set "ftp::ftp${ftp}(Output)" FTPLog
	ftp::Type $ftp binary
	if [ftp::Get $ftp $path $fn] {
	    CATVOTFile $fn
	}

	ftp::Close $ftp

	if [file exists $fn] {
	    catch {file delete -force $fn}
	}
    }
}

proc CATVOTHTTP {url} {
    global loadParam
    global ds9
    global http

    ParseURL $url r
    set fn "$ds9(tmpdir)/[file tail $r(path)]"

    set code 200
    set meta {}
    set mime {}
    set encoding {}

    set ch [open $fn w]
    set token [http::geturl $url -timeout $http(timeout) -channel $ch -binary 1 -headers "[ProxyHTTP]"]
    catch {close $ch}

    upvar #0 $token t

    # Code
    set code [http::ncode $token]

    # Meta
    set meta $t(meta)

    # Mime-type
    # we want to strip and extra info after ';'
    regexp -nocase {([^;])*} $t(type) mime

    HTTPLog $token
    http::cleanup $token

    global debug
    if {$debug(tcl,cat)} {
	puts "CATVOTHTTP: fn $fn : code $code : meta $meta : mime $mime : encoding $encoding"
    }

    CATVOTFile $fn

    if {[file exists $fn]} {
	catch {file delete -force $fn}
    }
}

