#  Copyright (C) 1999-2009
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc InitSAMP {} {
    global ds9

    SAMPConnect
    set ds9(samp,showError) 1
}

proc SAMPConnect {} {
    global ds9
    global samp
    global map

    if {!$ds9(samp)} {
	return
    }

    if {!$ds9(samp,status)} {
	catch {unset samp}

	if {![SAMPParseHub]} {
	    return
	}

	set rr {}

	# register
	set params [list "string $samp(secret)"]
	if {![SAMPSend {samp.hub.register} $params rr]} {
	    return
	}
	set rr [lindex $rr 1]
	foreach ff $rr {
	    foreach {key val} $ff {
		switch -- $key {
		    samp.hub-id {set samp(hub) $val}
		    samp.self-id {set samp(self) $val}
		    samp.private-key {set samp(private) $val}
		}
	    }
	}

	# declare metadata
	catch {unset map}
	set map(samp.name) {string "SAOImage DS9"}
	set map(samp.description.text) {string "SAOImage DS9 is an astronomical visualization application"}
	set map(samp.icon.url) {string "http://hea-www.harvard.edu/RD/ds9/sun.gif"}
	set map(samp.documentation.url) {string "http://hea-www.harvard.edu/RD/ds9/ref/index.html"}

	set map(home.page) {string "http://hea-www.harvard.edu/RD/ds9/"}
	set map(author.name) {string "William Joye"}
	set map(author.email) {string "saord@cfa.harvard.edu"}
	set map(author.affiliation) {string "Smithsonian Astrophysical Observatory"}
	set map(ds9.version) "string \"$ds9(version)\""

	set param1 [list "string $samp(private)"]
	set param2 [list "struct map"]
	set params "$param1 $param2"
	if {![SAMPSend {samp.hub.declareMetadata} $params rr]} {
	    return
	}

	# callback
	set param1 [list "string $samp(private)"]
	set param2 [list "string http://$samp(home)"]
	set params "$param1 $param2"
	if {![SAMPSend {samp.hub.setXmlrpcCallback} $params rr]} {
	    return
	}
	xmlrpc::serve $ds9(samp,port)

	# declare subscriptions
	catch {unset map}
	set map(samp.app.ping) {struct mapPing}
	set map(samp.hub.disconnect) {struct mapDisconnect}
	set map(samp.hub.event.shutdown) {struct mapShutdown}
	set map(samp.hub.event.register) {struct mapRegister}
	set map(samp.hub.event.unregister) {struct mapUnregister}
	set map(table.load.votable) {struct mapTableLoadVotable}
	set map(image.load.fits) {struct mapImageLoadFits}

	set mapPing(SAMPRcvdPing) {string {}}
	set mapDisconnect(SAMPRcvdDisconnect) {string {}}
	set mapShutdown(SAMPRcvdEventShutdown) {string {}}
	set mapRegister(SAMPRcvdEventRegister) {string {}}
	set mapUnregister(SAMPRcvdEventUnregister) {string {}}
	set mapTableLoadVotable(SAMPRcvdTableLoadVotable) {string {}}
	set mapImageLoadFits(SAMPRcvdImageLoadFits) {string {}}

	set param1 [list "string $samp(private)"]
	set param2 [list "struct map"]
	set params "$param1 $param2" 
	if {![SAMPSend {samp.hub.declareSubscriptions} $params rr]} {
	    return
	}

	set ds9(samp,status) 1
	UpdateFileMenu
    }
}

proc SAMPSend {method params resultVar} {
    upvar $resultVar result
    global samp
    global ds9

    global debug
    if {$debug(tcl,samp)} {
	puts "SAMPSend: $method $params"
    }

    if {[catch {set result [xmlrpc::call $samp(url) $method $params]}]} {
	if {$ds9(samp,showError)} {
	    Error "[msgcat::mc {SAMP Error: $result}]"
	}
	return 0
    }

    if {$debug(tcl,samp)} {
	puts "SAMPSend Result: $result"
    }
    return 1
}

proc SAMPReply {key status result} {
    global map
    global samp

    global debug
    if {$debug(tcl,samp)} {
	puts "SAMPReply: $key $status $result"
    }

    catch {unset map}
    set mapWarning(samp.warningtxt) "string \"$result\""
    set mapError(samp.errortxt) "string \"$result\""
    switch -- $status {
	OK {
	    set map(samp.status) {string "samp.ok"}
	    set map(samp.result) {struct mapWarning}
	}
	WARNING {
	    set map(samp.status) {string "samp.warning"}
	    set map(samp.result) {struct mapWarning}
	    set map(samp.error)  {struct mapError}
	}
	ERROR {
	    set map(samp.status) {string "samp.error"}
	    set map(samp.error)  {struct mapError}
	}
    }
    set param1 [list "string $samp(private)"]
    set param2 [list "string $key"]
    set param3 [list "struct map"]
    set params "$param1 $param2 $param3"
    if {![SAMPSend {samp.hub.reply} $params rr]} {
	return
    }
}

# receiveNotification(string sender-id, map message)
proc samp.client.receiveNotification {args} {
    global debug
    if {$debug(tcl,samp)} {
	puts "SAMP ReceivedNotification: $args"
    }
    set secret [lindex $args 0]
    set id [lindex $args 1]
    set map [lindex $args 2]

    set mtype [lindex $map 0]
    set params [lindex $map 1]

    set method [lindex $mtype 1]
    set args [lindex $params 1]

    if {$debug(tcl,samp)} {
	puts "SAMP Jump: $method $args"
    }
    switch -- $method {
	samp.app.ping {}
	samp.hub.disconnect {SAMPRcvdDisconnect}
	samp.hub.event.shutdown {SAMPRcvdEventShutdown}
	samp.hub.event.register {}
	samp.hub.event.unregister {}
	image.load.fits {SAMPRcvdImageLoadFits args}
	table.load.votable {SAMPRcvdTableLoadVotable args}
    }
    return {string OK}
}

# receiveCall(string sender-id, string msg-id, map message)
proc samp.client.receiveCall {args} {
    global debug
    if {$debug(tcl,samp)} {
	puts "SAMP ReceivedCall: $args"
    }

    set secret [lindex $args 0]
    set id [lindex $args 1]
    set key [lindex $args 2]
    set map [lindex $args 3]

    set mtype [lindex $map 0]
    set params [lindex $map 1]

    set method [lindex $mtype 1]
    set args [lindex $params 1]

    if {$debug(tcl,samp)} {
	puts "SAMP Jump: $method $args"
    }
    switch -- $method {
	image.load.fits {
	    SAMPRcvdImageLoadFits args
	    SAMPReply $key OK {}
	}
	table.load.votable {
	    SAMPRcvdTableLoadVotable args
	    SAMPReply $key OK {}
	}
	default {
	    SAMPReply $key ERROR "Unknown subscription: $method"
	}
    }
    return {string OK}
}

# receiveResponse(string responder-id, string msg-tag, map response)
proc samp.client.receiveResponse {args} {
    set key [lindex $args 0]
    set value [lindex $args 1]
    set map [lindex $args 2]

    return {string OK}
}

# support
proc SAMPParseHub {} {
    global ds9
    global samp

    # who are we
    if {[info hostname] != {}} {
	set samp(home) "[info hostname]:$ds9(samp,port)"
    } else {
	if {$ds9(samp,showError)} {
	    Error "[msgcat::mc {SAMP Internal Error}]"
	}
	return 0
    }

    set fn [file join [GetEnvHome] {.samp}]

    if {![file exist $fn]} {
	if {$ds9(samp,showError)} {
	    Error "[msgcat::mc {SAMP Internal Error}]"
	}
	return 0
    }

    set fp [open $fn r]
    while {1} {
	if {[gets $fp line] == -1} {
	    break
	}

	# skip any comments
	if {[string range $line 0 0] == "#"} {
	    continue;
	}

	if {[regexp -nocase {samp.secret=(.*)} $line foo ss]} {
	    set samp(secret) $ss
	}
	if {[regexp -nocase {samp.hub.xmlrpc.url=(.*)/(.*)} $line foo ss xx]} {
	    set samp(url) $ss
	    set samp(method) "$xx"
	}
    }
    catch {close $fp}

    if {$samp(secret) == {} || $samp(url) == {} || $samp(method) == {}} {
	if {$ds9(samp,showError)} {
	    Error "[msgcat::mc {SAMP Internal Error}]"
	}
	return 0
    }
    return 1
}

proc SAMPDisconnect {} {
    global ds9
    global samp

    if {$ds9(samp,status)} {
	# disconnect
	set params [list "string $samp(private)"]
	set rr {}
	SAMPSend {samp.hub.unregister} $params rr
	catch {unset samp}
	set ds9(samp,status) 0

	UpdateFileMenu
    }
}

proc SAMPInfo {} {
    global ds9
    global samp

    set rr {}
    if {$ds9(samp,status)} {
	append rr "[format "URL:\t%s" $samp(url)]\n"
	append rr "[format "HUB:\t%s" $samp(hub)]\n"
	append rr "[format "METHOD:\t%s" $samp(method)]\n"

	SimpleTextDialog samptxt "[msgcat::mc {SAMP Information}]" \
	    80 20 append bottom $rr
    } else {
	Error "[msgcat::mc {SAMP not connected}]"
    }
}

# CallBacks

proc SAMPRcvdDisconnect {} {
    global samp

    if {$ds9(samp,status)} {
	catch {unset samp}
	set ds9(samp,status) 0

	UpdateFileMenu
    }
}

proc SAMPRcvdEventShutdown {} {
    global ds9
    global samp

    if {$ds9(samp,status)} {
	catch {unset samp}
	set ds9(samp,status) 0

	UpdateFileMenu
    }
}

proc SAMPRcvdImageLoadFits {varname} {
    upvar $varname args

    global current
    global ds9

    set url {}
    set id {}
    set name {}

    foreach arg $args {
	foreach {key val} $arg {
	    switch -- $key {
		url {set url $val}
		image-id {set id $val}
		name {set name $val}
	    }
	}
    }

    if {$url != {}} {
	switch -- [$current(frame) get type] {
	    base -
	    3d {CreateFrame}
	    rgb {}
	}
	if {$ds9(display,user) != "tile"} {
	    set ds9(display,user) tile
	    DisplayMode
	}

	LoadURL $url
    }
}

proc SAMPRcvdTableLoadVotable {varname} {
    upvar $varname args

    set url {}
    set id {}

    foreach arg $args {
	foreach {key val} $arg {
	    switch -- $key {
		url {set url $val}
		table-id {set id $val}
	    }
	}
    }

    if {$url != {}} {
	CATVOTURL $url
    }
}

proc ProcessSAMPCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global ds9
    global env

    switch -- [string tolower [lindex $var $i]] {
	port {set ds9(samp,port) [lindex $var $i]}
	connect {
	    incr i
	    if {[FromYesNo [lindex $var $i]]} {
		SAMPConnect
	    } else {
		SAMPDisconnect
	    }
	}

	yes -
	true -
	on -
	1 -
	no -
	false -
	off -
	0 {set ds9(samp) [FromYesNo [lindex $var $i]]}
    }
}
